clear all;

%% Demo for detection and estimation of multivariate self-similar sources
%
%% If you use this code, please cite
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% P. Abry, H. Wendt, G. Didier, "Detecting and Estimating Multivariate
% Self-Similar Sources in High-Dimensional Noisy Mixtures,"	IEEE Workshop
% Statistical Signal Proces. (SSP), Freiburg, Germany, June 2018.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%--- Data parameters
M0 = 7; % embedding dimension
SNR=4.5;% Signal-to-Noise ration
rng(790904);

%--- Estimation parameters
Nwt=3; % Daubechies wavelet
j1=2; % scales j=j1:j2 for estimation
j2=6;
wtype=0; % weights for linear regresions (0: unweighted)

%--- Bootstrap parameters
NB=400; % number of bootstrap resamples
LB=10; % bootstrap block length
alpha=0.05; % significance for test

%--- Load bivariate example data
dd=load('bivar_ofbm_data.mat');data00=dd.S;N=dd.N;H1=dd.H1;H2=dd.H2;Rss=dd.Rss;clear dd;

%--- Embedd in high-dimensional noise
W = randn(M0,M0) ; W= W*W' ; W = W(:,1:2) ;
for m=1:size(W,1); W(m,:)=W(m,:)/sqrt(sum(W(m,:).^2)); end;
data0 = W*cumsum(data00,2) ; % Mixture
data = data0+randn(size(data0))*sqrt( 1/SNR ) ;

%--- Estimate
[est,nj,JM] = OFBM_estim_Jm(data,Nwt,j1,j2,wtype) ; Nj=nj.W;

LL=est.lambdaj';
figure(1);clf; plot(log2(LL)'); grid on; title('eigenanalysis');xlabel('j');

%--- Model Order Selection
JJ0=j1:j2; JJ1=1:JM;

% use scale invariance model to flatten eigenspectra across scales
tl=LL;
for k=1:size(tl,1);
    Hest(k)=est.lambda(k)-1/2; % OFBM scaling parameter estimate
    %TL(k,:)=(LL(k,:)-LL(k,j2)).*2.^(-2*(Hest(k)+1/2)*(JJ1-1)) + LL(k,j2);
    %TL(k,:)=LL(k,:);
    TL(k,:)=LL(k,:).*2.^(-2*(Hest(k)+1/2)*(JJ1-1));
end
%figure(2);clf; plot(log2(TL)'); grid on; title('eigenanalysis');xlabel('j');
for j=1:JM
    [J,JminR(:,j)] = ModelOrder(TL(:,j),Nj(j)) ;
end
[J,JminRS] = ModelOrder(sum(squeeze(TL(:,JJ0)),2),Nj(JJ0(1))) ;

ModelOrder_AIC=JminRS(1)
ModelOrder_AIC_majority=majorityvote(JminR(1,j1:j2))


figure(1);clf;
Pdat=cumsum(data00');for i=1:2; Pdat(:,i)=Pdat(:,i)-min(Pdat(:,i));Pdat(:,i)=Pdat(:,i)/max(Pdat(:,i)); end
plot(1:N,Pdat(:,1),'r',1:N,Pdat(:,2)-1,'b'); axis tight; 
title(['Original 2 OFBM components: (H_1,H_2)=(',num2str(H1),',',num2str(H2),'), \rho=',num2str(Rss)]);

figure(2);clf;hold on;
PdatW=data';
for i=1:size(data,1);
    PdatW(:,i)=PdatW(:,i)-min(PdatW(:,i));PdatW(:,i)=PdatW(:,i)/max(PdatW(:,i));
    plot(1:N,PdatW(:,i)-i,'k'); 
end
axis tight; hold off;
    
title([num2str(M0),' mixed components. Estimated number of true components = ',num2str(ModelOrder_AIC)]);

H=[H1 H2]
Hest=sort(Hest(end-ModelOrder_AIC+1:end))

