function [coef, leaders, nj] = DxLx1d(data, Nwt, gamint,symm)

%
%  [coef, leaders, nj] = DxLx1d_newNan_wavelab(data, Nwt [,gamint,sym])
%
% compute Dyadic wavelet and leader coefficients  
%
% gamint -- fractional integration of order gamint is applied
% sym=0  -- use Daubechies Wavelet of order Nwt.
% sym=1  -- use Daubechies Symmetrized Wavelet of order Nwt.
%
%%%
% SR, ens-Lyon, 11/2013
%


Norm=1;

%-- paramater for  fractional integration
if nargin<3; gamint=0; end;
if isempty(gamint); gamint=0; end;

if nargin<4; symm=0; end;
%-- Initialize the wavelet filters
n = length(data) ;         % data length
if symm == 0 % Daubechies Wavelet
    h = rlistcoefdaub(Nwt) ;   % filter
    nl = length(h) ;           % length of filter, store to manage edge effect later
    gg1 = -1*(-1).^(1:nl).*h ; % wavelet filter
    hh1=fliplr(h);             % scaling filter
    % parameter for the centering of the wavelet
    x0=2; 
    x0Appro=2*Nwt; 

else % Daubechies Symmetrized Wavelet
    Nwt=abs(Nwt);
    h = rlistcoefdaub(Nwt) ;   % filter
    nl = length(h) ;           % length of filter, store to manage edge effect later
    gg1c = -1*(-1).^(1:nl).*h ; % wavelet filter
    hh1c=fliplr(h);             % scaling filter
    tmp=conv(h,hh1c)/sqrt(2);
    nu=-2*Nwt+1;
%     [v,nv] = tildeurflippeur(-tmp,nu);
    [v,nv] = tildeurflippeur(tmp,nu);
    
    [hh1,nh1] = flippeur(tmp,nu) ; nl=length(hh1);
    [gg1,ng1] = flippeur(v,nv) ;   

    
    % parameter for the centering of the wavelet
    x0=2*Nwt; 
    x0Appro=2*Nwt; 

end

%--- Predict the max # of octaves available given Nwt, and take the min with
nbvoies= fix( log2(length(data)) );
% nbvoies = min( fix(log2(n/(2*Nwt+1)))  , nbvoies); %   safer, casadestime having problems
nbvoies = min( fix(log2(n/(nl+1)))  , nbvoies);

%--- Compute the WT, calculate statistics
approW=data;
for j=1:nbvoies         % Loop Scales
    
    %-- Phase 1a: get the wavelet coefficients/appro at this scale
    njtemp = length(approW) ;
    conv_gg1 = conv(approW,gg1) ; conv_gg1(isnan(conv_gg1))=Inf;
    conv_hh1 = conv(approW,hh1) ; conv_hh1(isnan(conv_hh1))=Inf;
      
    
    % border effect
    fp=nl-1; % index of first good value
    lp=njtemp; % index of last good value
    
    % replace border with Inf     
    conv_gg1(1:fp-1)=Inf;
    conv_gg1(lp+1:end)=Inf;
    
    conv_hh1(1:fp-1)=Inf;
    conv_hh1(lp+1:end)=Inf;

    %-- centering and decimation 
    approW=conv_hh1((1:2:njtemp)+x0Appro-1);
    decime=conv_gg1((1:2:njtemp)+x0-1);     
             
    %-- passage Norme L1
    sig=sign(decime);
    AbsdqkW = abs(decime)*2^(j/2)/2^(j/Norm);   
     
    %-- max before integration
    lesi=find(isfinite(AbsdqkW));
    
    %% HW
    if length(lesi)<3; return; end
    
    coef(j).supcoefnoint=max(AbsdqkW(lesi));
    
    %-- fractional integration and max
    AbsdqkW = AbsdqkW*2^(gamint*j);
    [coef(j).supcoef, coef(j).supcoefid]=max(AbsdqkW(lesi));
    
    %-- store results
    coef(j).allvalue=AbsdqkW;
    coef(j).allxpos=1:2^j:n;
    coef(j).value=AbsdqkW(lesi);
    coef(j).value_noabs=AbsdqkW(lesi).*sig(lesi);
    coef(j).xpos=coef(j).allxpos(lesi);
    coef(j).gamma=gamint;
    nj.W(j) = length(lesi);            % number of coefficients
    
   
    % for leaders 
    if j ==1
        %-- compute and store leaders sans voisin  
        leaders(j).sans_voisin.allvalue = AbsdqkW;
        leaders(j).sans_voisin.allxpos=coef(j).allxpos;
        
        %% useless
        leaders(j).sans_voisin.value = AbsdqkW(lesi);
        leaders(j).sans_voisin.xpos=coef(j).allxpos(lesi);
        %%
        
        %-- compute leaders  
        nc=floor(length(AbsdqkW)/2);
        voisin = max([AbsdqkW(1:end-2); AbsdqkW(2:end-1);AbsdqkW(3:end)]);
       
        
        %-- store results
        ifini=find(isfinite(voisin));
        leaders(j).allvalue=voisin;        
        leaders(j).allxpos=coef(j).allxpos(2:end-1);    
        leaders(j).value=voisin(ifini);
        leaders(j).xpos=leaders(j).allxpos(ifini);
        
        
        nj.L(j) = length(leaders(j).value); % number of leaders
 
    else
        %-- compute and store leaders sans voisin
        nc=floor(length(leaders(j-1).sans_voisin.allvalue)/2);
        leaders(j).sans_voisin.allvalue = max([coef(j).allvalue(1:nc); leaders(j-1).sans_voisin.allvalue(1:2:2*nc) ; leaders(j-1).sans_voisin.allvalue(2:2:2*nc)]);      
        leaders(j).sans_voisin.allxpos = coef(j).allxpos(1:nc);
  
        %% useless
        ifini=find(isfinite(leaders(j).sans_voisin.allvalue));
        leaders(j).sans_voisin.value = leaders(j).sans_voisin.allvalue(ifini);
        leaders(j).sans_voisin.xpos = leaders(j).sans_voisin.allxpos(ifini);
        %% 
        
        %-- compute leaders
        voisin = max([leaders(j).sans_voisin.allvalue(1:end-2);leaders(j).sans_voisin.allvalue(2:end-1);leaders(j).sans_voisin.allvalue(3:end)]);
        
        %-- store results
        ifini=find(isfinite(voisin));
        leaders(j).allvalue=voisin;
        leaders(j).allxpos=coef(j).allxpos(2:nc-1);
        leaders(j).value=voisin(ifini);
        leaders(j).xpos=leaders(j).allxpos(ifini);
         
        nj.L(j) = length(leaders(j).value); % number of leaders
        
    end
 
     leaders(j).gamma=gamint;    
     [leaders(j).mincoef, leaders(j).mincoefid]=min(leaders(j).value);
     [leaders(j).supcoefL, leaders(j).supcoefidL]=max(leaders(j).value);
     leaders(j).supcoefnoint=coef(j).supcoefnoint; leaders(j).supcoef=coef(j).supcoef; leaders(j).supcoefid=coef(j).supcoefid;
     coef(j).mincoef=leaders(j).mincoef; coef(j).mincoefid=leaders(j).mincoefid;
     coef(j).supcoefL=leaders(j).supcoefL; coef(j).supcoefidL=leaders(j).supcoefidL;
         

end

end


function h=rlistcoefdaub(regu)

if ~(regu<11 & regu>0),
	mess=[' Ondelette non implantee'];
	disp(mess);
	return;
else
	if regu==1,
   h=[1/sqrt(2) 1/sqrt(2)];
 %  g=[1/sqrt(2) -1/sqrt(2)];
end
if regu==2,
   h(1:2)=[0.482962913145 0.836516303738];
   h(3:4)=[0.224143868042 -0.129409522551];
end
if regu==3,
   h(1:2)=[0.332670552950 0.806891509311];
   h(3:4)=[0.459877502118 -0.135011020010];
   h(5:6)=[-0.085441273882 0.035226291882];
end
if regu==4,
   h(1:2)=[0.230377813309 0.714846570553];
   h(3:4)=[0.630880767930 -0.027983769417];
   h(5:6)=[-0.187034811719 0.030841381836];
   h(7:8)=[0.032883011667 -0.010597401785];
end
if regu==5,
   h(1:2)=[0.160102397974 0.603829269797];
   h(3:4)=[0.724308528438 0.138428145901];
   h(5:6)=[-0.242294887066 -0.032244869585];
   h(7:8)=[0.077571493840 -0.006241490213];
   h(9:10)=[-0.012580751999 0.003335725285];
end
if regu==6,
   h(1:2)=[0.111540743350 0.494623890398];
   h(3:4)=[0.751133908021 0.315250351709];
   h(5:6)=[-0.226264693965 -0.129766867567];
   h(7:8)=[0.097501605587 0.027522865530];
   h(9:10)=[-0.031582039318 0.000553842201];
   h(11:12)=[0.004777257511 -0.001077301085];
end
if regu==7,
   h(1:2)=[0.077852054085 0.396539319482];
   h(3:4)=[0.729132090846 0.469782287405];
   h(5:6)=[-0.143906003929 -0.224036184994];
   h(7:8)=[0.071309219267 0.080612609151];
   h(9:10)=[-0.038029936935 -0.016574541631];
   h(11:12)=[0.012550998556 0.000429577973];
   h(13:14)=[-0.001801640704 0.000353713800];
end
if regu==8,
   h(1:2)=[0.054415842243 0.312871590914];
   h(3:4)=[0.675630736297 0.585354683654];
   h(5:6)=[-0.015829105256 -0.284015542962];
   h(7:8)=[0.000472484574 0.128747426620];
   h(9:10)=[-0.017369301002 -0.044088253931];
   h(11:12)=[0.013981027917 0.008746094047];
   h(13:14)=[-0.004870352993 -0.000391740373];
   h(15:16)=[0.000675449406 -0.000117476784];
end
if regu==9,
   h(1:2)=[0.038077947364 0.243834674613];
   h(3:4)=[0.604823123690 0.657288078051];
   h(5:6)=[0.133197385825 -0.293273783279];
   h(7:8)=[-0.096840783223 0.148540749338];
   h(9:10)=[0.030725681479 -0.067632829061];
   h(11:12)=[0.000250947115 0.022361662124];
   h(13:14)=[-0.004723204758 -0.004281503682];
   h(15:16)=[0.001847646883 0.000230385764];
   h(17:18)=[-0.000251963189 0.000039347320];
end
if regu==10,
   h(1:2)=[0.026670057901 0.188176800078];
   h(3:4)=[0.52720118932 0.688459039454];
   h(5:6)=[0.281172343661 -0.249846424327];
   h(7:8)=[-0.195946274377 0.127369340336];
   h(9:10)=[0.093057364604 -0.071394147166];
   h(11:12)=[-0.029457536822 0.033212674059];
   h(13:14)=[0.003606553567 -0.010733175483];
   h(15:16)=[0.001395351747 0.001992405295];
   h(17:18)=[-0.000685856695 -0.000116466855];
   h(19:20)=[0.000093588670 -0.000013264203];
end
end
end
