

function [est,nj,JM] = OFBM_estim_Jm(data,Nwt,j1,j2,wtype) ; 
%% function [est,nj,JM] = OFBM_estim_Jm(data,Nwt,j1,j2,wtype) ; 
%
% Estimation of OFBM model

%% If you use this code, please cite
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% P. Abry, H. Wendt, G. Didier, "Detecting and Estimating Multivariate
% Self-Similar Sources in High-Dimensional Noisy Mixtures,"	IEEE Workshop
% Statistical Signal Proces. (SSP), Freiburg, Germany, June 2018.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gamint = 0.5 ; 
markersize = 15 ; 
markersize2 = 8 ; 
linewidth = 2 ; 
linewidth2 = 1 ; 
fontsize = 20 ;

[P,nn] = size(data) ; 
if P>nn
    tmp = nn ; 
    nn = P ;
    P = tmp ; 
end

for k = 1:1:P
    [cx,lx, nj] = DxLx1d(data(k,:),Nwt,gamint);
     NJ=nj.W;
     j2 = min(j2,length(NJ)) ; 
     WD{k} = cx ; 
end

JM=find(nj.W>=P); JM=JM(end); j2=min(j2,JM);
nj.W=nj.W(1:JM);
nj.L=nj.L(1:JM);
NJ=nj.W;

for k = 1:1:P
    for m = 1:1:P
        for j=1:length(NJ);
            tmp(j)=mean(real(WD{k}(j).value_noabs.*WD{m}(j).value_noabs));
            WW(k,m,j) = tmp(j) ; 
        end
    end
end

[a,b,J] = size(WW) ; 


for j = 1:JM
    %WW(:,:,j)=WW(:,:,j) +eye(size(WW(:,:,j)))*sqrt(norm(WW(:,:,j)))*1e-10;
    tmp2 = WW(:,:,j) ; 
    [Vtmp,Dtmp] = eig(tmp2) ;
    for k = 1:1:P
        lambda(j,k) = Dtmp(k,k) ;
        loglambda(j,k) = log2(Dtmp(k,k))/2/j ;
    end
    VV(:,:,j) = Vtmp ; 
end

for j = 1:JM
    tmp = WW(:,:,j) ; 
    [Vtmp,Dtmp] = eig(tmp) ;
    for k = 1:1:P
        lambda(j,k) = Dtmp(k,k) ;
    end
    VV(:,:,j) = Vtmp ; 
end

for k = 1:1:P
    for m = 1:1:P
        wavcov(k,m,:)= squeeze(WW(k,m,:))./sqrt(squeeze(WW(k,k,:)).*squeeze(WW(m,m,:))) ; 
    end
end
[a,b,J] = size(WW) ; 

JJ = 1:1:J ; 
JJJ = [j1:1:j2] ; 
est.lambdaj = lambda ; 
est.VVj = VV ;
est.wavcov = wavcov ; 
est.VV = squeeze(VV(:,:,J-2)) ; 
est.beta = est.VV(1,2)/est.VV(2,2) ; 
est.WW = WW ;
for j=1:1:J
    est.betaj(j) = VV(1,2,j)/VV(2,2,j) ;
end
est.beta2 = mean(est.betaj(J-1:J)) ;  
est.Nj = NJ ; 

for p=1:1:P
    % p
    % tmp = polyfit(JJJ,log2(lambda(JJJ,p)'),1) ; 
    tmp = log2(lambda(:,p)') ; 
     [slope,intercept] = SmartLDplot(tmp,NJ,ones(size(tmp)),j1,j2,wtype) ; 
    est.lambda(p) = (slope)/2 ;  
end

for k = 1:1:P
    for m = 1:1:P
        % [k m]
        tmp = log2(squeeze(abs(WW(k,m,:))))' ; 
%         pp = polyfit(JJJ,log2(tmp),1) ; 
%         est.h(k,m) = (pp(1)+1)/2 ; 
        [slope,intercept] = SmartLDplot(tmp,NJ,ones(size(tmp)),j1,j2,wtype) ; 
         est.h(k,m) = (slope)/2  ; 
    end
end

end


function [slope,intercept] = SmartLDplot(yj,nj,varyj,j1,j2,wtype) ; 

jj = j1:j2 ;
J  = length(jj); 
njj  = nj(jj);
yjj  = yj(jj);
varyjj= varyj(jj);

if     wtype==0  % uniform weights
   wvarjj = ones(1,J);   
   wstr = 'Uniform';
elseif wtype==1  % Gaussian type weights
   wvarjj = 1./njj;        
   wstr = 'Gaussian';
elseif  wtype==2   % weights from data 
   wvarjj = varyjj; 
   wstr = 'Estimated';
else % all other cases
   fprintf('** Weight option not recognised, using uniform weights\n')
   wvarjj = ones(1,J); 
   wstr = 'Uniform';
end

% use weighted regression formula in all cases
S0 = sum(1./wvarjj) ;
S1 = sum(jj./wvarjj) ;
S2 = sum(jj.^2./wvarjj) ;
wjj = (S0 * jj - S1) ./ wvarjj / (S0*S2-S1*S1) ;
vjj = (S2 - S1 * jj) ./ wvarjj / (S0*S2-S1*S1) ;


%  Estimate  zeta
slope  = sum(wjj .* yjj ) ;       % zeta is just the slope, unbiased regardless of the weights
intercept     = sum(vjj .* yjj ) ;       % intercept  'a'
end








 